Class: ClaudeAgentSDK::Configuration
- Inherits:
-
Object
- Object
- ClaudeAgentSDK::Configuration
- Defined in:
- lib/claude_agent_sdk/configuration.rb
Overview
Configuration class for setting default options
Use this to set default options that will be merged with every request. This is especially useful in Rails applications where you want to configure defaults once during initialization.
Instance Attribute Summary collapse
-
#default_options ⇒ Object
Returns the value of attribute default_options.
Instance Method Summary collapse
-
#initialize ⇒ Configuration
constructor
A new instance of Configuration.
Constructor Details
#initialize ⇒ Configuration
Returns a new instance of Configuration.
34 35 36 |
# File 'lib/claude_agent_sdk/configuration.rb', line 34 def initialize @default_options = {} end |
Instance Attribute Details
#default_options ⇒ Object
Returns the value of attribute default_options.
32 33 34 |
# File 'lib/claude_agent_sdk/configuration.rb', line 32 def @default_options end |