Class: ClaudeAgentSDK::TaskBudget
- Defined in:
- lib/claude_agent_sdk/types.rb
Overview
API-side task budget in tokens. When set, the model is made aware of its remaining token budget so it can pace tool use and wrap up before the limit.
Instance Attribute Summary collapse
-
#total ⇒ Object
Returns the value of attribute total.
Instance Method Summary collapse
Methods inherited from Type
#[], #[]=, from_hash, #initialize, wrap
Constructor Details
This class inherits a constructor from ClaudeAgentSDK::Type
Dynamic Method Handling
This class handles dynamic methods through the method_missing method in the class ClaudeAgentSDK::Type
Instance Attribute Details
#total ⇒ Object
Returns the value of attribute total.
1371 1372 1373 |
# File 'lib/claude_agent_sdk/types.rb', line 1371 def total @total end |
Instance Method Details
#to_h ⇒ Object
1373 1374 1375 |
# File 'lib/claude_agent_sdk/types.rb', line 1373 def to_h { total: @total } end |