Class: ClaudeAgentSDK::TaskBudget
- Inherits:
-
Object
- Object
- ClaudeAgentSDK::TaskBudget
- Defined in:
- lib/claude_agent_sdk/types.rb
Overview
API-side task budget in tokens. When set, the model is made aware of its remaining token budget so it can pace tool use and wrap up before the limit.
Instance Attribute Summary collapse
-
#total ⇒ Object
Returns the value of attribute total.
Instance Method Summary collapse
-
#initialize(total:) ⇒ TaskBudget
constructor
A new instance of TaskBudget.
- #to_h ⇒ Object
Constructor Details
#initialize(total:) ⇒ TaskBudget
Returns a new instance of TaskBudget.
1770 1771 1772 |
# File 'lib/claude_agent_sdk/types.rb', line 1770 def initialize(total:) @total = total end |
Instance Attribute Details
#total ⇒ Object
Returns the value of attribute total.
1768 1769 1770 |
# File 'lib/claude_agent_sdk/types.rb', line 1768 def total @total end |
Instance Method Details
#to_h ⇒ Object
1774 1775 1776 |
# File 'lib/claude_agent_sdk/types.rb', line 1774 def to_h { total: @total } end |