Class: ClaudeAgentSDK::ToolPermissionContext
- Inherits:
-
Object
- Object
- ClaudeAgentSDK::ToolPermissionContext
- Defined in:
- lib/claude_agent_sdk/types.rb
Overview
Tool permission context
Instance Attribute Summary collapse
-
#agent_id ⇒ Object
Returns the value of attribute agent_id.
-
#signal ⇒ Object
Returns the value of attribute signal.
-
#suggestions ⇒ Object
Returns the value of attribute suggestions.
-
#tool_use_id ⇒ Object
Returns the value of attribute tool_use_id.
Instance Method Summary collapse
-
#initialize(signal: nil, suggestions: [], tool_use_id: nil, agent_id: nil) ⇒ ToolPermissionContext
constructor
A new instance of ToolPermissionContext.
Constructor Details
#initialize(signal: nil, suggestions: [], tool_use_id: nil, agent_id: nil) ⇒ ToolPermissionContext
Returns a new instance of ToolPermissionContext.
686 687 688 689 690 691 |
# File 'lib/claude_agent_sdk/types.rb', line 686 def initialize(signal: nil, suggestions: [], tool_use_id: nil, agent_id: nil) @signal = signal @suggestions = suggestions @tool_use_id = tool_use_id # Unique ID for this tool call within the assistant message @agent_id = agent_id # Sub-agent ID if running within an agent context end |
Instance Attribute Details
#agent_id ⇒ Object
Returns the value of attribute agent_id.
684 685 686 |
# File 'lib/claude_agent_sdk/types.rb', line 684 def agent_id @agent_id end |
#signal ⇒ Object
Returns the value of attribute signal.
684 685 686 |
# File 'lib/claude_agent_sdk/types.rb', line 684 def signal @signal end |
#suggestions ⇒ Object
Returns the value of attribute suggestions.
684 685 686 |
# File 'lib/claude_agent_sdk/types.rb', line 684 def suggestions @suggestions end |
#tool_use_id ⇒ Object
Returns the value of attribute tool_use_id.
684 685 686 |
# File 'lib/claude_agent_sdk/types.rb', line 684 def tool_use_id @tool_use_id end |